import { createReadStream, createWriteStream } from 'node:fs'
import { readFile, stat, writeFile } from 'node:fs/promises'
import { pipeline } from 'node:stream/promises'
import { createBrotliCompress, createGzip } from 'node:zlib'
import type { Plugin } from 'vite'
import { glob } from 'glob'

export const PROJECT_NAME = '@open-xchange/vite-plugin-ox-bundle'

/**
 * This plugin creates bundles fom a list of css/js/svg files.
 *
 * The output bundle is a string with two delimiters '%&$1' and '%&$2'. First
 * delimiter splits between different files inside the string. The second delimiter
 * is used to split between fileName and content from a file.
 */
export default function pluginOxBundle (): Plugin {
  return {
    name: PROJECT_NAME,
    writeBundle: {
      order: 'post',
      sequential: true,
      async handler ({ dir }) {
        if (!dir) throw new Error('missing output directory')
        const data = await readFile(`${dir}/manifest.json`, 'utf8')
        const manifest = JSON.parse(data) as Record<string, unknown>

        // Add static files to manifest.json for warmup cache of ui-middleware
        const staticFiles = await glob(
          [
            `${dir}/**/*`
          ],
          {
            nodir: true
          }
        )

        for (const staticFile of staticFiles) {
          if (staticFile.endsWith('.map')) continue
          const manifestName = staticFile.replace(`${dir}/`, '')
          if (!manifest[manifestName]) {
            manifest[manifestName] = {
              src: manifestName,
              file: manifestName
            }
          }

          if (staticFile.endsWith('json') || staticFile.endsWith('mp3')) continue

          try {
            const { size } = await stat(staticFile)
            if (size > 600) {
              const readStream = createReadStream(staticFile)
              if (staticFile.endsWith('.html')) {
                const writeStream = createWriteStream(`${staticFile}.gz`)
                pipeline(readStream, createGzip(), writeStream).catch((err) => console.log(err))
              } else {
                const writeStream = createWriteStream(`${staticFile}.br`)
                pipeline(readStream, createBrotliCompress(), writeStream).catch((err) => console.log(err))
              }
            }
          } catch (err) {
            console.log(`Error during compression of ${staticFile}, `, err)
          }
        }

        await writeFile(`${dir}/manifest.json`, JSON.stringify(manifest))
      }
    }
  }
}
